import { e as LogHandlerOptions, b as LogHandler, L as LogLevel } from './log-94b534a3';

declare function csv(data: any[], headerRow: string[]): string;

declare function useBase(alphaOrBase: string | number): {
    encode: (source: number[] | Uint8Array | ArrayBuffer, padToLength?: number) => string;
    decode: (source: string, padToLength?: number) => Uint8Array;
};
declare const encodeBase16: (source: number[] | Uint8Array | ArrayBuffer, padToLength?: number) => string;
declare const decodeBase16: (source: string, padToLength?: number) => Uint8Array;
declare const encodeBase32: (source: number[] | Uint8Array | ArrayBuffer, padToLength?: number) => string;
declare const decodeBase32: (source: string, padToLength?: number) => Uint8Array;
declare const encodeBase58: (source: number[] | Uint8Array | ArrayBuffer, padToLength?: number) => string;
declare const decodeBase58: (source: string, padToLength?: number) => Uint8Array;
declare const encodeBase62: (source: number[] | Uint8Array | ArrayBuffer, padToLength?: number) => string;
declare const decodeBase62: (source: string, padToLength?: number) => Uint8Array;

declare const toCamelCase: (s: string) => string;
declare function toCapitalize(s: string): string;
declare function toCapitalizeWords(s: string): string;

declare function stringToBoolean(value?: string, defaultValue?: boolean): boolean;
declare function stringToInteger(value?: string, defaultValue?: number): number;
declare function stringToFloat(value?: string, defaultValue?: number): number;
declare function valueToBoolean(value?: any, defaultValue?: boolean): boolean;
declare function valueToInteger(value?: any, defaultValue?: number): number;
declare function valueToFloat(value?: any, defaultValue?: number): number;
declare function valueToString(value?: any, defaultValue?: string): string;
declare const toFloat: typeof valueToFloat;
declare const toInt: typeof valueToInteger;
declare const toString: typeof valueToString;
declare const toBool: typeof valueToBoolean;
declare type RenderMessagesOptions = {
    trace?: boolean;
    pretty?: boolean;
};
declare function formatMessages(messages: any[], opt?: RenderMessagesOptions): any[];
declare function renderMessages(messages: any[], opt?: RenderMessagesOptions): string;

declare const escapeHTML: (s: string) => string;
declare const unescapeHTML: (s: string) => string;

declare type EntryProcessor = (key: string, value: any) => any;
/**
 * Similar to JSON.stringify but can handle circular references
 */
declare function jsonStringify(obj: any, replacer?: EntryProcessor | null, spaces?: string | number | null, cycleReplacer?: EntryProcessor): string;

declare function parseOrderby(value?: string): {
    field: string;
    orderby: string;
    asc: boolean;
    desc: boolean;
};
declare function composeOrderby(field: string, asc?: boolean): string;
declare function cmp(a: any, b: any, asc?: boolean): number;
declare function sortedOrderby<T>(values: T[], ...orderby: string[]): T[];

declare function toValidFilename(string: string): string;

declare function escapeRegExp(value: RegExp | string): string;

interface SortableItem {
    sort_weight: number;
}
declare function startSortWeight(items: SortableItem[]): number;
declare function endSortWeight(items: SortableItem[]): number;
declare function moveSortWeight(newIndex: number, oldIndex: number, items: SortableItem[]): number;
declare function sortedItems<T extends SortableItem>(items: T[]): T[];

declare function linkifyPlainText(text: string): string;
declare function toHumanReadableUrl(url: string): string;

declare function size(obj: any): any;
declare function last<T>(array?: T[]): T | undefined;
declare function empty(value: any): boolean;
declare function cloneObject<T>(obj: T): T;

declare function arrayUnique<T>(x: T[]): T[];
declare function arrayMinus<T>(x: T[], y: T[]): T[];
declare function arrayUnion<T>(...a: T[][]): T[];
/** `[1,[2,3]]` becomes `[1,2,3]` */
declare function arrayFlatten<T>(list: (T | T[])[]): T[];
declare function arrayIntersection<T>(x: T[], y: T[]): T[];
declare function arraySymmetricDifference<T>(x: T[], y: T[]): T[];
declare function arrayRemoveElement<T>(arr: T[], el: T): T[];
/** Only have it once in the set */
declare function arraySetElement<T>(arr: T[], el: T): T[];
declare function arrayFilterInPlace<T>(array: T[], fn: (el: T) => boolean): T[];
declare function arrayToggleInPlace<T>(array: T[], el: T): T[];
declare function arrayEmptyInPlace<T>(array: T[]): T[];
declare function arraySorted<T>(arr: Iterable<T> | ArrayLike<T>, cond?: ((a: T, b: T) => number) | undefined): T[];
declare function arraySortedNumbers(arr: number[]): number[];
declare function arrayIsEqual<T>(array1: T[], array2: T[]): boolean;
declare function arrayShuffleInPlace<T>(array: T[]): T[];
declare function arrayShuffle<T>(array: T[]): T[];
/** Randomly shuffle the order of the array's elements. Force to have a different order if array has more than one element. */
declare function arrayShuffleForce<T>(array: T[]): T[];
declare function arrayRandomElement<T>(array: T[]): T;

declare const isObject: (obj: any) => any;
declare const isPrimitive: (obj: any) => boolean;
declare function deepEqual(a: any, b: any, hash?: WeakSet<object>): boolean;
declare function deepMerge(target: any, ...sources: any[]): any;

declare global {
    interface ZeedGlobalContext {
    }
}
declare function getGlobalContext(): ZeedGlobalContext;

/**
 * Very basic logger. Please take a look at the browser and node
 * optimized loggers. This one is just the absolute fallback option.
 *
 * @param level Log level
 * @returns Logger
 */
declare function LoggerConsoleHandler(opt?: LogHandlerOptions): LogHandler;

interface NamespaceFilter {
    (name: string): boolean;
    accept: RegExp[];
    reject: RegExp[];
    filter: string;
}
/**
 * Filter as described here https://github.com/visionmedia/debug#wildcards
 *
 * @param filter Namespace filter
 * @returns Function to check if filter applies
 */
declare function useNamespaceFilter(filter?: string): NamespaceFilter;

declare function getStackLlocationList(stack: string): any[];
/**
 * Get the source code location of the caller
 * https://stackoverflow.com/a/47296370/140927
 *
 * @param level Number of levels to go down the stack trace
 * @param stripCwd Strip the current working directory, only reasonable for Node.js environment
 * @returns
 */
declare function getSourceLocation(level?: number, stripCwd?: boolean): string;

declare type EmitterHandler = (...objs: any[]) => void;
declare type EmitterAllHandler<T = string> = (key: T, ...objs: any[]) => void;
declare type ListenerSignature<L> = {
    [E in keyof L]: (...args: any[]) => any;
};
declare type DefaultListener = {
    [k: string]: (...args: any[]) => any;
};
declare class Emitter<L extends ListenerSignature<L> = DefaultListener> {
    subscribers: any;
    subscribersOnAny: any[];
    call: L;
    emit<U extends keyof L>(event: U, ...args: Parameters<L[U]>): Promise<boolean>;
    onAny(fn: EmitterHandler): void;
    on<U extends keyof L>(event: U, listener: L[U]): {
        cleanup: () => void;
    };
    onCall(handlers: Partial<L>): void;
    once<U extends keyof L>(event: U, listener: L[U]): void;
    off<U extends keyof L>(event: U, listener: L[U]): this;
    removeAllListeners(event?: keyof L): this;
}
declare global {
    interface ZeedGlobalContext {
        emitter?: Emitter;
    }
    interface ZeedGlobalEmitter {
    }
}
/** Global emitter that will listen even across modules */
declare function getGlobalEmitter(): Emitter<ZeedGlobalEmitter>;
declare const messages: Emitter<DefaultListener>;
declare function lazyListener(emitter: any, listenerKey?: string): (key?: string, skipUnmatched?: boolean) => Promise<any>;

/** See http://developer.mozilla.org/en-US/docs/Web/API/MessageEvent */
interface ChannelMessageEvent {
    data: any;
    origin?: string;
    lastEventId?: string;
}
/**
 * Inspired by
 * http://developer.mozilla.org/en-US/docs/Web/API/BroadcastChannel
 * https://deno.com/deploy/docs/runtime-broadcast-channel
 * */
declare abstract class Channel extends Emitter<{
    message(event: ChannelMessageEvent): void;
    messageerror(event: ChannelMessageEvent): void;
    connect(): void;
    disconnect(): void;
    close(): void;
}> {
    id: string;
    abstract isConnected?: boolean;
    abstract postMessage(data: any): void;
    close(): void;
}
/** Very basic channel demonstrating local communication */
declare class LocalChannel extends Channel {
    isConnected: boolean;
    other?: LocalChannel;
    postMessage(data: any): void;
}
declare function fakeWorkerPair(): [LocalChannel, LocalChannel];

interface Disposable {
    cleanup(): Promise<void>;
}
declare type Json = string | number | boolean | null | {
    [property: string]: Json;
} | Json[];
interface ObjectStorage {
    setItem(key: string, value: Json): void;
    getItem(key: string): Json | undefined;
    removeItem(key: string): void;
    clear(): void;
    allKeys(): string[];
}

interface Encoder {
    encode(data: Json): ArrayBuffer;
    decode(data: ArrayBuffer): Json;
}
declare class JsonEncoder implements Encoder {
    encode(data: Json): ArrayBuffer;
    decode(data: ArrayBuffer): Json;
}

declare type MessageAction = {
    name: string;
    id: string;
    args?: Json[];
};
declare type MessageResult = {
    id: string;
    result?: Json;
    error?: {
        stack?: string;
        name: string;
        message: string;
    };
};
declare type Message = MessageAction | MessageResult;
declare type MessagesOptions = {
    timeout?: number;
};
declare type MessagesDefaultMethods<L> = {
    connect?(channel: Channel): void;
    options(opt: MessagesOptions): L;
};
declare type MessagesMethods<L> = L & MessagesDefaultMethods<L>;
declare function useMessages<L extends object>(opt?: {
    channel?: Channel;
    encoder?: Encoder;
    handlers?: L;
    retryAfter?: number;
    ignoreUnhandled?: boolean;
}): MessagesMethods<L>;
declare function useMessageHub(opt?: {
    channel?: Channel;
    encoder?: Encoder;
    retryAfter?: number;
    ignoreUnhandled?: boolean;
}): {
    connect: (newChannel: Channel) => void;
    server<L extends object>(newHandlers: L): void;
    client<L_1 extends object>(): MessagesMethods<L_1>;
};

declare type Mutex = (fn: () => void, elseFn?: () => void) => boolean;
declare function createMutex(): Mutex;

declare function getWindow(): any | undefined;
declare function getNavigator(): any | undefined;
declare function getGlobal(): any;
declare function detect(info?: {
    ios: boolean;
    macos: boolean;
    windows: boolean;
    beaker: boolean;
    electron: boolean;
    wkwebview: boolean;
    pwa: boolean;
    pwaInstalled: boolean;
    browser: boolean;
    node: boolean;
    worker: boolean;
    jest: boolean;
    macosNative: boolean;
    iosNative: boolean;
    appleNative: boolean;
    touch: boolean;
}): {
    ios: boolean;
    macos: boolean;
    windows: boolean;
    beaker: boolean;
    electron: boolean;
    wkwebview: boolean;
    pwa: boolean;
    pwaInstalled: boolean;
    browser: boolean;
    node: boolean;
    worker: boolean;
    jest: boolean;
    macosNative: boolean;
    iosNative: boolean;
    appleNative: boolean;
    touch: boolean;
};
declare const isBrowser: () => boolean;
declare const platform: {
    ios: boolean;
    macos: boolean;
    windows: boolean;
    beaker: boolean;
    electron: boolean;
    wkwebview: boolean;
    pwa: boolean;
    pwaInstalled: boolean;
    browser: boolean;
    node: boolean;
    worker: boolean;
    jest: boolean;
    macosNative: boolean;
    iosNative: boolean;
    appleNative: boolean;
    touch: boolean;
};
/**
 * Before closing the tab/window or quitting the node process
 * allow to do something important first
 */
declare function useExitHandler(handler: () => void): void;

/** Sleep for `milliSeconds`. Example 1s: `await sleep(1000)` */
declare function sleep(milliSeconds: number): Promise<void>;
declare function immediate(): Promise<void>;
declare const timeoutReached: unique symbol;
declare function timeout<T>(promise: Promise<T>, milliSeconds: number, timeoutValue?: symbol): Promise<T | typeof timeoutValue>;
declare const timoutError: Error;
declare function isTimeout(value: any): boolean;
declare function tryTimeout<T>(promise: Promise<T>, milliSeconds: number): Promise<T | undefined>;
/** Wait for `event` on `obj` to emit. Resolve with result or reject on `timeout` */
declare function waitOn(obj: any, event: string, timeoutMS?: number): Promise<any>;
declare function isPromise<T>(value: Promise<T> | T): value is Promise<T>;
/** This is exactly what Prose.resolve(x) is supposed to be: return a Promise no matter what type x is */
declare function promisify<T>(value: Promise<T> | T): Promise<T>;

declare type QueueTask<T = any> = () => Promise<T>;
/** Guarentee serial execution of tasks. Able to wait, pause, resume and cancel all. */
declare class SerialQueue {
    private queue;
    private isPaused;
    private waitToFinish;
    private currentTask?;
    private log;
    name: string;
    constructor(opt?: {
        name?: string;
        logLevel?: LogLevel;
    });
    private performNext;
    /** Enqueue task to be executed when all other tasks are done. Except `immediate = true`. */
    enqueue<T>(task: QueueTask<T>, opt?: {
        immediate?: boolean;
        name?: string;
    }): Promise<T>;
    /** If a task is already performing, execute immediately. Otherwise enqueue as usual. */
    enqueueReentrant<T>(task: QueueTask<T>, opt?: {
        name?: string;
    }): Promise<T>;
    /** Remove all tasks from queue that are not yet executing. */
    cancelAll(unblock?: boolean): Promise<void>;
    /** Pause execution after current task is finished. */
    pause(): Promise<void>;
    /** Resume paused queue. */
    resume(): void;
    /** Wait for all tasks to finish */
    wait(): Promise<unknown>;
}

interface MemStorageOptions {
    objectFromString?: (data: string) => any;
    objectToString?: (data: any) => string;
}
declare class MemStorage implements ObjectStorage {
    private store;
    private pretty;
    private objectFromString;
    private objectToString;
    constructor(opt?: MemStorageOptions);
    setItem(key: string, value: Json): void;
    getItem(key: string): Json | undefined;
    removeItem(key: string): void;
    clear(): void;
    allKeys(): string[];
}

interface DebounceOptions {
    delay?: number;
    noTrailing?: boolean;
    debounceMode?: boolean;
}
declare type DebounceFunction = Function & {
    cancel: () => void;
};
declare function throttle(callback: Function, opt?: DebounceOptions): DebounceFunction;
declare function debounce(callback: Function, opt?: DebounceOptions): DebounceFunction;
declare function throttleAnimationFrame(callback: Function): DebounceFunction;

/**
 * @returns Timestamp in miliseconds
 */
declare const getTimestamp: () => number;
declare function formatMilliseconds(ms: number): string;

declare function uuid(): string;
declare function uuidB32(): string;
declare function uname(name?: string): string;
declare function qid(): string;
declare const uuidv4: () => string;
declare function suidBytes(): Uint8Array;
declare function suid(): string;
declare function suidDate(id: string): Date;
declare function suidBytesDate(id: Uint8Array): Date;

export { arrayRemoveElement as $, escapeHTML as A, unescapeHTML as B, jsonStringify as C, parseOrderby as D, composeOrderby as E, cmp as F, sortedOrderby as G, toValidFilename as H, escapeRegExp as I, startSortWeight as J, endSortWeight as K, moveSortWeight as L, sortedItems as M, linkifyPlainText as N, toHumanReadableUrl as O, size as P, last as Q, RenderMessagesOptions as R, SortableItem as S, empty as T, cloneObject as U, arrayUnique as V, arrayMinus as W, arrayUnion as X, arrayFlatten as Y, arrayIntersection as Z, arraySymmetricDifference as _, encodeBase32 as a, MemStorage as a$, arraySetElement as a0, arrayFilterInPlace as a1, arrayToggleInPlace as a2, arrayEmptyInPlace as a3, arraySorted as a4, arraySortedNumbers as a5, arrayIsEqual as a6, arrayShuffleInPlace as a7, arrayShuffle as a8, arrayShuffleForce as a9, Message as aA, MessagesOptions as aB, MessagesDefaultMethods as aC, MessagesMethods as aD, useMessages as aE, useMessageHub as aF, Mutex as aG, createMutex as aH, getWindow as aI, getNavigator as aJ, getGlobal as aK, detect as aL, isBrowser as aM, platform as aN, useExitHandler as aO, sleep as aP, immediate as aQ, timeoutReached as aR, timeout as aS, timoutError as aT, isTimeout as aU, tryTimeout as aV, waitOn as aW, isPromise as aX, promisify as aY, SerialQueue as aZ, MemStorageOptions as a_, arrayRandomElement as aa, isObject as ab, isPrimitive as ac, deepEqual as ad, deepMerge as ae, getGlobalContext as af, LoggerConsoleHandler as ag, useNamespaceFilter as ah, getStackLlocationList as ai, getSourceLocation as aj, ChannelMessageEvent as ak, Channel as al, LocalChannel as am, fakeWorkerPair as an, EmitterHandler as ao, EmitterAllHandler as ap, ListenerSignature as aq, DefaultListener as ar, Emitter as as, getGlobalEmitter as at, messages as au, lazyListener as av, Encoder as aw, JsonEncoder as ax, MessageAction as ay, MessageResult as az, decodeBase32 as b, throttle as b0, debounce as b1, throttleAnimationFrame as b2, getTimestamp as b3, formatMilliseconds as b4, Disposable as b5, Json as b6, ObjectStorage as b7, uuid as b8, uuidB32 as b9, uname as ba, qid as bb, uuidv4 as bc, suidBytes as bd, suid as be, suidDate as bf, suidBytesDate as bg, csv as c, decodeBase16 as d, encodeBase16 as e, encodeBase58 as f, decodeBase58 as g, encodeBase62 as h, decodeBase62 as i, toCapitalize as j, toCapitalizeWords as k, stringToInteger as l, stringToFloat as m, valueToInteger as n, valueToFloat as o, valueToString as p, toFloat as q, toInt as r, stringToBoolean as s, toCamelCase as t, useBase as u, valueToBoolean as v, toString as w, toBool as x, formatMessages as y, renderMessages as z };
